#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "barrier4xi.py">
#<abstract4xi abstract = "provides Barrier4Xi class"/>
#<refer4xi reference = "synchronize.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------
#

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Barrier4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
    import struct
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # imported-as ones 
    from time import time as _time, sleep as _sleep        
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from bufferwrapper4xi import BufferWrapper4Xi
from condition4xi import Condition4Xi
from lock4xi import Lock4Xi
from reason_object4xi import reason_object4xi
from register_after_fork4xi import register_after_fork4xi
from sense_object4xi import sense_object4xi
from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi    
#"
#/>

                

# =================================================================================================
#<class4xi class = "Barrier4Xi" base = "threading.Barrier" child = "None" synopsis = "offers barrier services" usage = "
#    managers.py:
#        SyncManager.register('Barrier', threading.Barrier, BarrierProxy)
# 
#    syncmanager4xi.py:
#        SyncManager4Xi.register('Barrier4Xi', threading.Barrier, BarrierProxy4Xi)"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
      
class Barrier4Xi(threading.Barrier):
    '''
    Barrier4Xi class offers barrier services  
    '''
  
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('sense', 'reason', 'ace')         
    #"
    #/>
                                     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Barrier4Xi" usage = "initializes with(self, parties, action=None, timeout=None)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, parties, action=None, timeout=None):
        '''
        Barrier4Xi method __init__: initializes with (self, parties, action=None, timeout=None)
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------
        class2print = "Barrier4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/4a) started" + "; init-of-Barrier4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        
        message2print = "{(Step 1/4b) borning" + "; barrier2xi-of-Barrier4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)

        message2print = "{(Step 1/4c) inheriting" + "; barrier-of-threading-inheriting-by-barrier2xi-of-Barrier4Xi: " + str(threading.Barrier)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        message2print = "{(Step 1/4d) passed" + "; parties-passed-to-init-of-Barrier4Xi: " + str(parties) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
        
        message2print = "{(Step 1/4e) passed" + "; action-passed-to-init-of-Barrier4Xi: " + str(action) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
        
        message2print = "{(Step 1/4f) passed" + "; timeout-passed-to-init-of-Barrier4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #    
                        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------
        wrapper2xi = BufferWrapper4Xi(struct.calcsize('i') * 2)

        message2print = "{(Step 2/4) set" + "; wrapper2xi-set-by-init-of-Barrier4Xi to: " + str(wrapper2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------
        cond2xi = Condition4Xi()
        
        message2print = "{(Step 3/4) set" + "; cond2xi-set-by-init-of-Barrier4Xi to: " + str(cond2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------
        self.__setstate__((parties, action, timeout, cond2xi, wrapper2xi))
        message2print = "{(Step 4/4a) performed" + "; setstate-of-barrier2xi-performed-by-init-of-Barrier4Xi to: " + str(self.__setstate__)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        self._state = 0
        message2print = "{(Step 4/4b) set" + "; state-of-barrier2xi-set-by-init-of-Barrier4Xi to: " + str(self._state)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        self._count = 0
        message2print = "{(Step 4/4c) set" + "; count-of-barrier2xi-set-by-init-of-Barrier4Xi to: " + str(self._count)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
                
        message2print = "{(Step 4/4d) returning" + "; init-of-Barrier4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                        
                
                                   
    #</methodize4xi>
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "__setstate__-of-Barrier4Xi" usage = "sets state with (self, state)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __setstate__(self, state):
        '''
        Barrier4Xi method __setstate__: sets state with (self, state)
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------    
        class2print = "Barrier4Xi"
        method2print = "__setstate__"
        
        message2print = "{(Step 1/3a) started" + "; setstate-of-Barrier4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

        message2print = "{(Step 1/3b) passed; state-passed-to-setstate-of-Barrier4Xi: " + str(state) + ";time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------        
        (self._parties, self._action, self._timeout,
         self._cond, self._wrapper) = state
         
         
        self._array = self._wrapper.create_memoryview().cast('i')

        message2print = "{(Step 2/3a) set" + "; array-of-barrier2xi-set-by-init-of-Barrier4Xi to: " + str(self._array)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------                     
        message2print = "{(Step 3/3) returning" + "; setstate-of-Barrier4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                        
          
    #</methodize4xi>
        
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "__getstate__-of-Barrier4Xi" usage = "gets state with (self, state)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __getstate__(self, state):
        '''
        Barrier4Xi method __getstate__: gets state with (self, state)
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/2)
        # -----------------------------------------------------------------------------------------    
        class2print = "Barrier4Xi"
        method2print = "__getstate__"
        
        message2print = "{(Step 1/2a) started" + "; getstate-of-Barrier4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

        message2print = "{(Step 1/2b) passed; state-passed-to-getstate-of-Barrier4Xi: " + str(state) + ";time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    

        # -----------------------------------------------------------------------------------------
        # (Step 2/2)
        # -----------------------------------------------------------------------------------------                     
        message2print = "{(Step 2/2a) returning" + "; parties-returning-by-getstate-of-Barrier4Xi: " + str(self._parties)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        message2print = "{(Step 2/2b) returning" + "; action-returning-by-getstate-of-Barrier4Xi: " + str(self._action)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        message2print = "{(Step 2/2c) returning" + "; timeout-returning-by-getstate-of-Barrier4Xi: " + str(self._timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        message2print = "{(Step 2/2d) returning" + "; cond-returning-by-getstate-of-Barrier4Xi: " + str(self._cond)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        message2print = "{(Step 2/2e) returning" + "; wrapper-returning-by-getstate-of-Barrier4Xi: " + str(self._wrapper)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        
        return (self._parties, self._action, self._timeout,
                self._cond, self._wrapper)
                        
          
    #</methodize4xi> 
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "state-of-Barrier4Xi" usage = "state property">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @property
    def _state(self):
        return self._array[0]
    #</methodize4xi>
    
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "state-of-Barrier4Xi" usage = "state property setter">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @_state.setter
    def _state(self, value):
        self._array[0] = value
    #</methodize4xi>
    
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "state-of-Barrier4Xi" usage = "count property">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @property
    def _count(self):
        return self._array[1]
    #</methodize4xi>
    
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "state-of-Barrier4Xi" usage = "count property setter">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @_count.setter
    def _count(self, value):
        self._array[1] = value
    #</methodize4xi>            
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Barrier4Xi" usage = "senses Barrier4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses Barrier4Xi class objects
        '''
          
        
        class2print = 'Barrier4Xi'
        method2print = 'sense'
        
        message2print = "{started" + "; sense-of-Barrier4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; barrier4xi-sensing-by-sense-of-Barrier4Xi: " + str(Barrier4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-Barrier4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['inheritance-of-barrier2xi'] = object
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-Barrier4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
      
        message2print = "{sensed" + "; barrier4xi-sensed-by-sense-of-Barrier4Xi: " + str(Barrier4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-Barrier4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Barrier4Xi" usage = "reasons Barrier4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons Barrier4Xi class objects
        '''
                
        
        class2print = 'Barrier4Xi'
        method2print = 'reason'

        message2print = "{started" + "; reason-of-Barrier4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Barrier4Xi-reasoning-by-reason-of-Barrier4Xi: " + str(Barrier4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-Barrier4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; Barrier4Xi-reasoned-by-reason-of-Barrier4Xi: " + str(Barrier4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-Barrier4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                              
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Barrier4Xi" usage = "aces Barrier4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces Barrier4Xi class objects 
        '''
                
        
        
        class2print = 'Barrier4Xi'
        method2print = 'ace'
        
        message2print = "{started" + "; ace-of-Barrier4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Barrier4Xi-acing-by-ace-of-Barrier4Xi: " + str(Barrier4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                    
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-Barrier4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; Barrier4Xi-aced-by-ace-of-Barrier4Xi: " + str(Barrier4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-Barrier4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                           
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of barrier4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

